#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Created on Sat Oct 24 12:10:14 2020

@author: Martín Carlos Araya <martinaraya@gmail.com>
"""

__all__ = ['WrongUnits', 'WrongValue', 'NoConversionFound']
__version__ = '0.0.2'
__release__ = 20220819

class WrongUnits(Exception):
    def __init__(self, message='unit not listed in library, unit must be a string.'):
        print('ERROR: Wrong Units, ' + message)


class WrongValue(Exception):
    def __init__(self, message='value unit must be a float or integer.'):
        print('ERROR: Wrong Value, ' + message)
        

class NoConversionFound(Exception):
    def __init__(self, message='for the provided units.'):
        print('ERROR: Conversion path not found, ' + message)


#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Created on Sat Oct 24 14:34:59 2020

@author: Martín Carlos Araya <martinaraya@gmail.com>
"""

__version__ = '0.1.1'
__release__ = 20220803


import numpy as _np
from ._errors import WrongUnits as _WrongUnits, WrongValue as _WrongValue
from ._operations import unitProduct, unitDivision, unitBasePower
from ._convert import converter as _converter, convertible as _convertible


class _units(object):
    """
    A class to cope with values associated to units, its arithmetic and logic
    operations and conversions.
    """

    def __init__(self):
        self.unit = None
        self.value = None
        self.name = None
        self.kind = None

    def __call__(self):
        return self.value

    def __repr__(self):
        return str(self.value) + '_' + str(self.unit)
    def __str__(self) :
        return str(self.value) + '_' + str(self.unit)

    def convert(self, newunit):
        if type(newunit) is not str:
            try:
                newunit = newunit.unit
            except:
                raise _WrongUnits()
        return self.kind(_converter(self.value, self.unit, newunit), newunit)

    def to(self,newunit):
        return self.convert(newunit)

    def __neg__(self):
        return self.kind(self.value * -1, self.unit)

    def __bool__(self):
        from .units.custom import userUnits
        from .units.unitless import dimensionless
        if self.kind in [dimensionless, userUnits]:
            return False
        return True

    def __abs__(self):
        return self.kind(abs(self.value), self.unit)

    def __add__(self,other):
        from .units.unitless import dimensionless
        if '.units.' in str(type(other)):
            if other.kind is self.kind:
                if self.unit == other.unit:
                    return self.kind(self.value + other.value, self.unit)
                else:
                    return self.kind(self.value + _converter(other.value, other.unit, self.unit), self.unit)
            if other.kind is dimensionless:
                return self.kind(self.value + other.value * self.value, self.unit)
            else : # add different units
                return (self, other)
        elif type(other) is tuple:
            result, flag = [], False
            for each in other:
                if type(each) is type(self):
                    result += [self + each]
                    flag = True
                else:
                    result += each
            return tuple(result) if flag else other + (self,)
        else:
            return self.kind(self.value + other, self.unit)
    def __radd__(self,other):
        return self.__add__(other)

    def __mul__(self,other):
        from .units.unitless import dimensionless
        from .units.define import units
        if '.units.' in str(type(other)):
            if self.kind is dimensionless:
                return other.kind(self.value * other.value, other.unit)
            elif other.kind is dimensionless:
                return self.kind(self.value * other.value, self.unit)
            elif self.unit == other.unit:
                return units(self.value * other.value, unitProduct(self.unit, other.unit))
            elif _convertible(other.unit, self.unit):
                return units(self.value * _converter(other.value, other.unit, self.unit), unitProduct(self.unit, other.unit))
            elif _convertible(unitBasePower(self.unit)[0], unitBasePower(other.unit)[0]):
                factor = _converter(1, unitBasePower(other.unit)[0], unitBasePower(self.unit)[0])
                return units(self.value * other.value * factor, unitProduct(self.unit,other.unit))
            else:
                return units(self.value * other.value,unitProduct(self.unit,other.unit))
        elif type(other) is tuple:
            result = []
            for each in other:
                result += [self * each]
            return tuple(result)
        else:
            return self.kind(self.value * other,self.unit)

    def __rmul__(self,other):
        return self.__mul__(other)

    def __pow__(self,other):
        from .units.unitless import dimensionless
        from .units.define import units
        if '.units.' in str(type(other)):
            if self.kind is dimensionless:
                return other.kind(self.value ** other.value, other.unit)
            elif other.kind is dimensionless:
                return self.kind(self.value ** other.value, self.unit)
            elif self.unit == other.unit:
                return units(self.value ** other.value, self.unit + '^' + other.unit)
            elif _convertible(other.unit,self.unit):
                return units(self.value ** _converter(other.value, other.unit, self.unit), self.unit + '^' + self.unit)
            elif _convertible(unitBasePower(self.unit)[0], unitBasePower(other.unit)[0]):
                factor = _converter(1, unitBasePower(other.unit)[0], unitBasePower(self.unit)[0])
                return units(self.value ** (other.value * factor), self.unit+'^'+other.unit)
            else:
                return units(self.value ** other.value, self.unit+'^'+other.unit)
        elif type(other) is tuple:
            result = []
            for each in other:
                result += [self ** each]
            return tuple(result)
        else:
            powunits = unitBasePower(self.unit)[1] * other
            if powunits == 0:
                powunits = 'dimensionless'
            elif powunits == 1:
                powunits = unitBasePower(self.unit)[0]
            else:
                powunits = unitBasePower(self.unit)[0] + str(powunits)
            return units(self.value ** other, powunits)

    def __sub__(self, other):
        return self.__add__(other * -1)

    def __rsub__(self, other):
        return -self + other

    def __truediv__(self, other):
        from .units.unitless import dimensionless
        from .units.define import units
        if '.units.' in str(type(other)):
            if self.kind is dimensionless:
                return other.kind(self.value / other.value, other.unit)
            elif other.kind is dimensionless :
                return self.kind(self.value / other.value, self.unit)
            elif self.unit == other.unit:
                return units(self.value / other.value, unitDivision(self.unit, other.unit))
            elif _convertible(other.unit, self.unit):
                return units(self.value / _converter(other.value, other.unit, self.unit), unitDivision(self.unit, other.unit))
            elif _convertible(unitBasePower(self.unit)[0], unitBasePower(other.unit)[0]):
                factor = _converter(1, unitBasePower(other.unit)[0], unitBasePower(self.unit)[0])
                return units(self.value / (other.value * factor), unitDivision(self.unit, other.unit))
            else:
                return units(self.value / other.value, unitDivision(self.unit, other.unit))
        elif type(other) is tuple:
            result = []
            for each in other:
                result += [self / each]
            return tuple(result)
        else:
            return units(self.value / other, self.unit)
    def __rtruediv__(self, other):
        from .units.define import units
        return units( other / self.value, self.unit + '-1' )

    def __floordiv__(self, other):
        from .units.unitless import dimensionless
        from .units.define import units
        if '.units.' in str(type(other)):
            if self.kind is dimensionless:
                return other.kind(self.value // other.value, other.unit)
            elif other.kind is dimensionless:
                return self.kind(self.value // other.value, self.unit)
            elif self.unit == other.unit:
                return units(self.value // other.value, unitDivision(self.unit, other.unit))
            elif _convertible(other.unit, self.unit):
                return units(self.value // _converter(other.value, other.unit, self.unit), unitDivision(self.unit, other.unit))
            elif _convertible(unitBasePower(self.unit)[0], unitBasePower(other.unit)[0]):
                factor = _converter(1, unitBasePower(other.unit)[0], unitBasePower(self.unit)[0])
                return units(self.value // (other.value * factor), unitDivision(self.unit, other.unit))
            else:
                return units(self.value // other.value, unitDivision(self.unit, other.unit))
        elif type(other) is tuple:
            result = []
            for each in other:
                result += [self / each]
            return tuple(result)
        else:
            return self.kind(self.value / other, self.unit)
    def __rfloordiv__(self, other):
        from .units.define import units
        return units( other // self.value, self.unit+'-1' )

    def __mod__(self, other):
        from .units.define import units
        if '.units.' in str(type(other)):
            from .units.unitless import dimensionless
            if self.kind is dimensionless:
                return other.kind(self.value % other.value, self.unit)
            elif other.kind is dimensionless:
                return self.kind(self.value % other.value, self.unit)
            elif self.unit == other.unit:
                return units(self.value % other.value, self.unit)
            elif _convertible(other.unit, self.unit):
                return units(self.value % _converter(other.value, other.unit, self.unit), self.unit)
            elif _convertible(unitBasePower(self.unit)[0], unitBasePower(other.unit)[0]):
                factor = _converter(1, unitBasePower(other.unit)[0], unitBasePower(self.unit)[0])
                return units(self.value % (other.value * factor), self.unit)
            else:
                return units(self.value % other.value, self.unit)
        elif type(other) is tuple:
            result = []
            for each in other:
                result += [self % each]
            return tuple(result)
        else:
            return self.kind(self.value % other,self.unit)

    def __lt__(self, other):
        if type(self) == type(other):
            return self.value < other.convert(self.unit).value
        else:
            msg = "'<' not supported between instances of '" +   (str(type(self))[str(type(self)).index("'")+1:len(str(type(self))) - str(type(self))[::-1].index("'")-1]).replace('__main__.','')   + "' and '" +   (str(type(other))[str(type(other)).index("'")+1:len(str(type(other))) - str(type(other))[::-1].index("'")-1]).replace('__main__.', '')   + "'"
            raise TypeError(msg)

    def __le__(self, other):
        if type(self) == type(other):
            return self.value <= other.convert(self.unit).value
        else :
            msg = "'<=' not supported between instances of '" +   (str(type(self))[str(type(self)).index("'")+1:len(str(type(self))) - str(type(self))[::-1].index("'")-1]).replace('__main__.','')   + "' and '" +   (str(type(other))[str(type(other)).index("'")+1:len(str(type(other))) - str(type(other))[::-1].index("'")-1]).replace('__main__.','')   + "'"
            raise TypeError(msg)

    def __eq__(self, other):
        if type(self) == type(other):
            return self.value == other.convert(self.unit).value
        else:
            msg = "'==' not supported between instances of '" +   (str(type(self))[str(type(self)).index("'")+1:len(str(type(self))) - str(type(self))[::-1].index("'")-1]).replace('__main__.','')   + "' and '" +   (str(type(other))[str(type(other)).index("'")+1:len(str(type(other))) - str(type(other))[::-1].index("'")-1]).replace('__main__.','')   + "'"
            raise TypeError(msg)

    def __ne__(self, other):
        if type(self) == type(other):
            return self.value != other.convert(self.unit).value
        else:
            msg = "'!=' not supported between instances of '" +   (str(type(self))[str(type(self)).index("'")+1:len(str(type(self))) - str(type(self))[::-1].index("'")-1]).replace('__main__.','')   + "' and '" +   (str(type(other))[str(type(other)).index("'")+1:len(str(type(other))) - str(type(other))[::-1].index("'")-1]).replace('__main__.','')   + "'"
            raise TypeError(msg)

    def __ge__(self, other):
        if type(self) == type(other):
            return self.value >= other.convert(self.unit).value
        else :
            msg = "'>=' not supported between instances of '" +   (str(type(self))[str(type(self)).index("'")+1:len(str(type(self))) - str(type(self))[::-1].index("'")-1]).replace('__main__.','')   + "' and '" +   (str(type(other))[str(type(other)).index("'")+1:len(str(type(other))) - str(type(other))[::-1].index("'")-1]).replace('__main__.','')   + "'"
            raise TypeError(msg)
    def __gt__(self, other):
        if type(self) == type(other):
            return self.value > other.convert(self.unit).value
        else :
            msg = "'>' not supported between instances of '" +   (str(type(self))[str(type(self)).index("'")+1:len(str(type(self))) - str(type(self))[::-1].index("'")-1]).replace('__main__.','')   + "' and '" +   (str(type(other))[str(type(other)).index("'")+1:len(str(type(other))) - str(type(other))[::-1].index("'")-1]).replace('__main__.','')   + "'"
            raise TypeError(msg)

    def __len__(self):
        try:
            return len(self.value)
        except:
            return 1

    def __getitem__(self, item):
        if type(item) is int:
            if item >= len(self):
                raise IndexError
        else:
            raise ValueError
        return self.value[item]

    def __iter__(self):
        if type(self.value) in (int, float):
            return _np.array((self.value,)).__iter__()
        else:
            return self.value.__iter__()

    # def __next__(self) :
    #     pass

    def getUnit(self):
        return self.unit

    def getValue(self):
        return self.value

    def checkValue(self,value):
        if type(value) in (list, tuple):
            return _np.array(value)
        elif type(value) in (int, float):
            return value
        elif type(value) is _np.ndarray:
            return value
        # elif type(value) == pandas.core.frame.DataFrame :
        #     return value
        else:
            raise _WrongValue()

    def checkUnit(self,units):
        if type(units) is not str:
            try:
                units = units.unit
            except:
                raise _WrongUnits
        if units in self.kind.classUnits:
            return units
        else:
            raise _WrongUnits


#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Created on Sat Oct 24 17:52:34 2020

@author: Martín Carlos Araya <martinaraya@gmail.com>
"""

__all__ = ['BFS']
__version__ = '0.1.0'
__release__ = 20220524


def BFS(graph, start, end, toPrint=False):
    """Assumes graph is a Digraph; start and end are nodes
        Returns a shortest path from start to end in graph"""
    initPath = [start]
    pathQueue = [initPath]
    visited = []
    while len(pathQueue) != 0:
        #Get and remove oldest element in pathQueue
        tmpPath = pathQueue.pop(0)
        if tmpPath in visited :
            if toPrint:
                print(' <UnitsConv> ' + str(len(pathQueue)) + ' paths in queue. ' + 'Already visited BFS path:\n', printPath(tmpPath))
        else :
            if toPrint:
                print(' <UnitsConv> ' + str(len(pathQueue)) + ' paths in queue. ' + 'Current BFS path:\n', printPath(tmpPath))
            lastNode = tmpPath[-1]
            if lastNode == end:
                return tmpPath
            for nextNode in graph.childrenOf(lastNode):
                if nextNode not in tmpPath:
                    newPath = tmpPath + [nextNode]
                    pathQueue.append(newPath)
            visited.append(tmpPath)
    return None


def printPath(path):
    """Assumes path is a list of nodes"""
    result = '  '
    if len(path) == 1 :
        result = result + str(path[0]) + ' = ' + str(path[0])
    else :
        for i in range(len(path)):
            if type(path[i]) == str :
                result = result + ' ' +  path[i] + ' '
            else:
                result = result + str(path[i])
                if i != len(path) - 1:
                    result = result + ' ' + chr(11157) + ' '
    return result


#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Created on Sat Oct 24 14:34:59 2020

@author: Martín Carlos Araya <martinaraya@gmail.com>
"""
__version__ = '0.1.1'
__release__ = 20220803

from .._dictionaries import dictionary
from .._unit import _units


class rate(_units):
    classUnits = dictionary['rate']
    def __init__(self, value, units):
        self.name = 'rate'
        self.kind = rate
        self.value = self.checkValue(value)
        self.unit = self.checkUnit(units)

class speed(_units):
    classUnits = dictionary['speed']
    def __init__(self, value, units):
        self.name = 'speed'
        self.kind = speed
        self.value = self.checkValue(value)
        self.unit = self.checkUnit(units)

def velocity(value, units):
    return speed(value, units)


#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Created on Sat Oct 24 14:34:59 2020

@author: Martín Carlos Araya <martinaraya@gmail.com>
"""
__version__ = '0.1.1'
__release__ = 20220803

from .._dictionaries import dictionary
from .._unit import _units


class density(_units):
    classUnits = dictionary['density']
    def __init__(self, value, units):
        self.name = 'density'
        self.kind = density
        self.value = self.checkValue(value)
        self.unit = self.checkUnit(units)


class volumeRatio(_units):
    classUnits = dictionary['volumeRatio']
    def __init__(self, value, units):
        self.name = 'volumeRatio'
        self.kid = volumeRatio
        self.value = self.checkValue(value)
        self.unit = self.checkUnit(units)


# class productivityIndex(_units):
#     classUnits = dictionary['productivityIndex']
#     def __init__(self,value,units) :
#         self.name = 'productivityIndex'
#         self.kind = productivityIndex
#         self.value = self.checkValue(value)
#         self.unit = self.checkUnit(units)

# class pressureGradient(_units):
#     classUnits = dictionary['pressureGradient']
#     def __init__(self,value,units) :
#         self.name = 'pressureGradient'
#         self.kind = pressureGradient
#         self.value = self.checkValue(value)
#         self.unit = self.checkUnit(units)




#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Created on Sat Oct 24 14:34:59 2020

@author: Martín Carlos Araya <martinaraya@gmail.com>
"""
__version__ = '0.1.1'
__release__ = 20220803

from .._dictionaries import dictionary
from .._unit import _units


class temperature(_units):
    classUnits = dictionary['temperature']
    def __init__(self, value, units):
        self.name = 'temperature'
        self.kind = temperature
        self.value = self.checkValue(value)
        self.unit = self.checkUnit(units)


#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Created on Sat Oct 24 14:34:59 2020

@author: Martín Carlos Araya <martinaraya@gmail.com>
"""
__version__ = '0.1.1'
__release__ = 20220803
__all__ = ['customUnits', 'userUnits']

from .._dictionaries import dictionary
from .._unit import _units


def customUnits(value, units):
    return userUnits(value,units)

class userUnits(_units):
    classUnits = dictionary['userUnits']
    def __init__(self, value, units):
        self.name = 'userUnits'
        self.kind = userUnits
        units = units.strip()
        if units not in dictionary['userUnits']:
            dictionary['userUnits'].append(units)
        self.value = self.checkValue(value)
        self.unit = self.checkUnit(units)


#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Created on Sat Oct 24 18:24:20 2020

@author: Martín Carlos Araya <martinaraya@gmail.com>
"""

__version__ = '0.2.1'
__release__ = 20220822

# from .units.custom import *
from .units.define import units
from ._convert import convertUnit
from ._convert import convertUnit as convert
from ._convert import convertUnit2 as convert2
from ._database import unitsNetwork

def print_path(switch):
    if type(switch) is str and switch.lower().strip() == 'off':
        switch = False
    unitsNetwork.print = bool(switch)


#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Created on Sat Oct 24 14:38:58 2020

@author: Martín Carlos Araya <martinaraya@gmail.com>
"""

__all__ = ['unitProduct','unitDivision']
__version__ = '0.1.3'
__release__ = 20220819

from ._dictionaries import dictionary
from ._convert import convertible
from ._helpers import isnumber


def unitBasePower(unit):
    """

    Parameters
    ----------
    unit : str
        a string representing the unit


    Returns
    -------
        tuple of base and power

    """
    uBas, uPow = '', ''
    oth = ''
    inv = False
    if '/' in unit and len(unit.split('/')) == 2 and isnumber(unit.split('/')[0]):
        invPow, unit = unit.split('/')
        inv = True

    for c in unit :
        if c.isdigit() :
            uPow += oth + c
            oth = ''
        elif c in ['-','+','.']:
            oth += c
        else :
            uBas += oth + c
            oth = ''
    uPow = 1 if uPow == '' else float(uPow) if '.' in uPow else int(uPow)
    if inv :
        uPow = uPow * -1 * (float(invPow) if '.' in invPow else int(invPow))
    return uBas, uPow


def unitBase(unit):
    """
    

    Parameters
    ----------
    unit : TYPE
        DESCRIPTION.

    Returns
    -------
    None.

    """
    return unitBasePower(unit)[0]


def unitProduct(unit1,unit2):
    """

    Parameters
    ----------
    unit1 :

    unit2 :


    Returns
    -------

    """

    if unit1 is None:
        unit1 = 'dimensionless'
    if unit2 is None:
        unit2 = 'dimensionless'

    if type(unit1) is str and len(unit1.strip(' ()')) == 0:
        unit1 = 'dimensionless'
    if type(unit2) is str and len(unit2.strip(' ()')) == 0:
        unit2 = 'dimensionless'

    if type(unit1) is str and len(unit1.split('/')) == 2 and unit1.split('/')[0] == unit1.split('/')[1]:
        unit1 = 'dimensionless'
    if type(unit2) is str and len(unit2.split('/')) == 2 and unit2.split('/')[0] == unit2.split('/')[1]:
        unit2 = 'dimensionless'

    if unit2.lower().strip(' ()') in dictionary['dimensionless']:
        return unit1
    if unit1.lower().strip(' ()') in dictionary['dimensionless']:
        if unit2.lower().strip(' ()') not in dictionary['dimensionless']:
            return unit2
        else :
            return unit1

    if unit1 != unit2 and convertible(unit1, unit2):
        return unitProduct(unit1, unit1)

    U1bas, U1pow = unitBasePower(unit1)
    U2bas, U2pow = unitBasePower(unit2)

    if convertible(U1bas, U2bas):
        Upow = U1pow+U2pow
        if Upow == -1:
            result = U1bas+'-1'
        elif Upow == 1:
            result = U1bas
        elif Upow == 0:
            result = U1bas + '/' + U1bas
        else :
            for c in ['+','-','^']:  # '*','/'
                if c in U1bas:
                    U1bas = '('+U1bas+')'
                    break
            result = U1bas + str(Upow)

    else:
        for c in ['+','-','^']:  # '*','/'
            if c in unit1:
                unit1 = '('+unit1+')'
                break
        for c in ['+','-','^']:  # '*','/'
            if c in unit2:
                unit2 = '('+unit2+')'
                break
        result = unit1 + '*' + unit2

    return result


def unitDivision(unit1, unit2):
    """

    Parameters
    ----------
    unit1 :

    unit2 :


    Returns
    -------

    """

    if unit1 is None:
        unit1 = 'dimensionless'
    if unit2 is None:
        unit2 = 'dimensionless'

    if type(unit1) is str and len(unit1.strip(' ()')) == 0:
        unit1 = 'dimensionless'
    if type(unit2) is str and len(unit2.strip(' ()')) == 0:
        unit2 = 'dimensionless'

    if type(unit1) is str and len(unit1.split('/')) == 2 and unit1.split('/')[0] == unit1.split('/')[1]:
        unit1 = 'dimensionless'
    if type(unit2) is str and len(unit2.split('/')) == 2 and unit2.split('/')[0] == unit2.split('/')[1]:
        unit2 = 'dimensionless'

    if unit2.lower().strip(' ()') in dictionary['dimensionless']:
        return unit1
    if unit1.lower().strip(' ()') in dictionary['dimensionless']:
        if unit2.lower().strip(' ()') not in dictionary['dimensionless']:
            uBas, uPow = unitBasePower('1/' + unit2)
            return uBas+str(uPow)
        else:
            return unit1

    if unit1 != unit2 and convertible(unit1, unit2):
        return unitDivision(unit1, unit1)

    U1bas, U1pow = unitBasePower(unit1)
    U2bas, U2pow = unitBasePower(unit2)

    if convertible(U1bas, U2bas):
        Upow = U1pow - U2pow
        if Upow == -1:
            result = U1bas + '-1'
        elif Upow == 1:
            result = U1bas
        elif Upow == 0:
            result = U1bas + '/' + U1bas
        else:
            for c in ['+','-','^']:  # '*','/'
                if c in U1bas:
                    U1bas = '('+U1bas+')'
                    break
            result = U1bas + str(Upow)

    elif ('+' not in unit1 and '-' not in unit1 and '^' not in unit1) and (
            '+' not in unit2 and '-' not in unit2 and '^' not in unit2):
        if '*' in unit1 and unitBase(unit2) in map(unitBase, unit1.split('*')):
            result = ''
            for u in unit1.split('*'):
                if unit2 == u:
                    return '*'.join([u for u in unit1.split('*') if u != unit2])
                elif unitBase(unit2) == unitBase(u):
                    result = (result + '*' + unitDivision(u, unit2).strip('*')).strip('*')
                else:
                    result = (result + '*' + u).strip('*')
    else:
        for c in ['+','-','^']:  # '*','/'
            if c in unit1:
                unit1 = '('+unit1+')'
                break
        for c in ['+','-','^']:  # '*','/'
            if c in unit2:
                unit2 = '('+unit2+')'
                break
        result = unit1 + '/' + unit2

    return result


#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Created on Sat Oct 24 14:34:59 2020

@author: Martín Carlos Araya <martinaraya@gmail.com>
"""
__version__ = '0.1.1'
__release__ = 20220803

from .._dictionaries import dictionary
from .custom import userUnits
from .force import pressure, weight, compressibility
from .geometry import length, area, volume
from .temperature import temperature
from .time import time
from .unitless import dimensionless
from .ratios import density, volumeRatio
from .rates import rate, speed


def units(value, units=None):
    if units is None and '.units.' in str(type(value)):
        value, units = value.value, value.units
    if units is None:
        units = 'dimensionless'
    if type(units) is not str:
        raise TypeError("'units' must be a string.")
    units = units.strip()
    for kind in dictionary:
        if units in dictionary[kind]:
            return eval( kind + "(" + str(value) + ",'" + units + "')" )
    return userUnits(value, units)

